/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;

import org.objectweb.jasmine.util.JasmineWrapperUtil;

/**
 * @author Julien Legrand
 *
 */
public abstract class AbstractConfiguratorComposite implements
        ConfiguratorComposite {

    /**
     *
     */
    protected Map<String, Configurator> configurators = new HashMap<String, Configurator>();

    // ------------------------------------------------------------------------
    // Implementation of the ConfiguratorComposite interface
    // ------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.objectweb.jasmine.configurator.ConfiguratorComposite#addConfigurator(java.lang.String, org.objectweb.jasmine.configurator.Configurator)
     */
    public void addConfigurator(String prefix, Configurator configurator) {
        configurators.put(prefix, configurator);
    }

    /* (non-Javadoc)
     * @see org.objectweb.jasmine.configurator.ConfiguratorComposite#removeConfigurator(org.objectweb.jasmine.configurator.Configurator)
     */
    public void removeConfigurator(Configurator configurator) {
        configurators.remove(configurator);
    }

    // ------------------------------------------------------------------------
    // Implementation of the Configurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#configure()
     */
    public void configure() throws ConfigurationException {

        Set<String> prefixes = configurators.keySet();

        for (String prefix : prefixes) {
            configurators.get(prefix).configure();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#getProperties()
     */
    public Properties getProperties() {

        Properties properties = new Properties();

        Set<String> prefixes = configurators.keySet();

        for (String prefix : prefixes) {
            JasmineWrapperUtil.concatProperties(properties, configurators.get(
                    prefix).getProperties());
        }

        return properties;

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#getProperty(java.lang.String)
     */
    public Object getProperty(String key) {

        StringTokenizer tokenizer = new StringTokenizer(key, "-");

        if (tokenizer.hasMoreTokens()) {

            String prefix = tokenizer.nextToken();

            /*
             * if property name is prefixed, delegate register
             */
            if (configurators.containsKey(prefix)) {
                return configurators.get(prefix).getProperty(key);
            }
        }

        return null;

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#setProperty(java.lang.String,
     *      java.lang.Object)
     */
    public boolean setProperty(String key, Object value) {

        /*
         * pre-processing on value
         */
        Object val = null;
        val = value;

        if (value instanceof String) {

            String sValue = (String) value;

            if (sValue.matches("true") || sValue.matches("false")) {
                val = Boolean.parseBoolean(sValue);
            }
            if (sValue.matches("\\d+")) {
                val = Integer.parseInt(sValue);
            }

        }

        StringTokenizer tokenizer = new StringTokenizer(key, "-");

        if (tokenizer.hasMoreTokens()) {

            String prefix = tokenizer.nextToken();

            /*
             * if property name is prefixed, delegate register
             */
            if (configurators.containsKey(prefix)) {
                configurators.get(prefix).setProperty(key, val);
                return true;
            }
        }

        return false;
    }

    // ------------------------------------------------------------------------
    // Overriden methods
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        for (Object o : this.getProperties().keySet()) {
            buffer.append(this.getProperties().get(o));
            buffer.append("\n");
        }
        return buffer.toString();
    }

}
