/**
 * JASMINe
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.objectweb.jasmine.configurator.ConfigurationException;

/**
 * The Idea : all configuration file are generated by velocity We have one
 * generator by configuration file.
 *
 * Default values are done in velocity file. We can push out default value
 * programmatically. Value pushed programmatically have priority. If value is
 * not done programmatically we use default value defined in velocity file. At
 * each configuration default values is restored.
 *
 * TODO : clear context at each generation
 *
 * <p>
 * contribution : Julien Legrand
 * </p>
 *
 * @author Yann Davin <yann.davin@gmail.com>
 */
public abstract class AbstractVelocityConfigurator implements
        VelocityConfigurator {

    /**
     *
     */
    protected VelocityEngine ve = null;

    /**
     *
     */
    protected VelocityContext context = null;

    /**
     *
     */
    protected Template template = null;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public AbstractVelocityConfigurator() {

        ve = new VelocityEngine();
        context = new VelocityContext();
        context.put("this", this);
    }

    public AbstractVelocityConfigurator(ClassLoader cl) {

        try {

            ve = new VelocityEngine();
            context = new VelocityContext();
            context.put("this", this);

            ve.setProperty("resource.loader", "class");
            ve
                    .setProperty("class.resource.loader.class",
                            "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
            if (cl != null)
                ve.setProperty("class.resource.loader.classloader", cl);

            ve.init();

        } catch (ResourceNotFoundException rnfe) {
            System.out.println("cannot find template " + getTemplateFile());
        } catch (ParseErrorException pee) {
            System.out.println("Syntax error in template " + getTemplateFile()
                    + ":" + pee);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public AbstractVelocityConfigurator(VelocityContext vcontext, ClassLoader cl) {

        try {

            ve = new VelocityEngine();
            context = new VelocityContext(vcontext);
            context.put("this", this);

            ve.setProperty("resource.loader", "class");
            ve
                    .setProperty("class.resource.loader.class",
                            "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
            if (cl != null)
                ve.setProperty("class.resource.loader.classloader", cl);

            ve.init();

        } catch (ResourceNotFoundException rnfe) {
            System.out.println("cannot find template " + getTemplateFile());
        } catch (ParseErrorException pee) {
            System.out.println("Syntax error in template " + getTemplateFile()
                    + ":" + pee);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ------------------------------------------------------------------------
    // Implementation of the Configurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#configure()
     */
    public void configure() throws ConfigurationException {

        try {

            write(getGeneratedFileName());

        } catch (ResourceNotFoundException e) {
            throw new ConfigurationException(e);
        } catch (ParseErrorException e) {
            throw new ConfigurationException(e);
        } catch (Exception e) {
            throw new ConfigurationException(e);
        }

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#put(java.lang.String,
     *      java.lang.Object)
     */
    public boolean setProperty(String key, Object value) {
        context.put(key, value);
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#get(java.lang.String)
     */
    public Object getProperty(String key) {
        return context.get(key);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.Configurator#getProperties()
     */
    public Properties getProperties() {

        Properties properties = new Properties();

        for (Object key : context.getKeys()) {

            properties.put(key, context.get(key.toString()));

        }

        return properties;
    }

    // ------------------------------------------------------------------------
    // Implementation of the VelocityConfigurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.VelocityConfigurator#getContext()
     */
    public VelocityContext getContext() {
        return this.context;
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * Write generated configuration in the file "filename"
     *
     * @param filename
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private void write(String filename) throws ResourceNotFoundException,
            ParseErrorException, Exception {

        File file = new File(filename);
        if(!file.exists())
            file.createNewFile();

        BufferedWriter out = new BufferedWriter(new FileWriter(filename));
        out.write(generate());
        out.close();
    }

    /**
     * @return
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private String generate() throws ResourceNotFoundException,
            ParseErrorException, Exception {

        template = ve.getTemplate(getTemplateFile());

        StringWriter writer = new StringWriter();

        if (template != null)
            template.merge(context, writer);

        return new String(writer.getBuffer());

    }

}
