/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.configurator.statik.velocity.templategenerator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * This class generates a Velocity template file, used by JASMINe configurator,
 * from an XML property file.
 *
 * TODO : This class can be optimized (file generation, needs SAX & DOM (???),
 * ...)
 *
 * @author Julien Legrand
 *
 */
public class XMLTemplateGenerator extends AbstractTemplateGenerator implements
        ContentHandler {

    /**
     *
     */
    private final static String XML_FILE_EXTENSION = ".xml";

    /**
     *
     */
    private File templateFile = null;

    /**
     *
     */
    private StringBuffer buffer = null;

    /**
     *
     */
    private Document domDocument;

    /**
     *
     */
    private Node domRoot = null;

    /**
     *
     */
    private Node currentNode = null;

    /**
     *
     */
    private final static int START_ELEMENT = 0;

    /**
     *
     */
    private final static int END_ELEMENT = 1;

    /**
     *
     */
    private int lastOp = 0;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public XMLTemplateGenerator(String prefix) {
        super(prefix);
    }

    // ------------------------------------------------------------------------
    // Implementation of the TemplateGenerator interface
    // ------------------------------------------------------------------------

    public void generate(File file) throws Exception {

        /*
         * create file to generate
         */
        String path = file.getAbsolutePath();
        templateFile = new File(path.replace(XML_FILE_EXTENSION,
                TEMPLATE_EXTENSION));

        /*
         * Initialize DOM document which will be used to generate the template
         * file
         */
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        factory.setIgnoringElementContentWhitespace(true);

        DocumentBuilder builder = factory.newDocumentBuilder();
        builder.setEntityResolver(new EntityResolver() {
            public InputSource resolveEntity(String publicId, String systemId)
                    throws SAXException, IOException {
                if (systemId.endsWith(".dtd"))
                    // this deactivates all DTDs by giving empty XML docs
                    return new InputSource(
                            new ByteArrayInputStream(
                                    "<?xml version='1.0' encoding='UTF-8'?>"
                                            .getBytes()));
                else
                    return null;
            }
        });

        domDocument = builder.parse(file);
        domRoot = domDocument.getDocumentElement();

        /*
         * Initialize the SAX parser which will be used to find properties
         */
        XMLReader saxReader = XMLReaderFactory.createXMLReader();
        saxReader.setContentHandler(this);
        saxReader.setEntityResolver(new EntityResolver() {
            public InputSource resolveEntity(String publicId, String systemId)
                    throws SAXException, IOException {
                if (systemId.endsWith(".dtd"))
                    // this deactivates all DTDs by giving empty XML docs
                    return new InputSource(
                            new ByteArrayInputStream(
                                    "<?xml version='1.0' encoding='UTF-8'?>"
                                            .getBytes()));
                else
                    return null;
            }
        });
        saxReader.parse(new InputSource(new FileInputStream(file)));

    }

    // ------------------------------------------------------------------------
    // Implementation of the ContentHandler interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException {
        buffer = new StringBuffer();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#endDocument()
     */
    public void endDocument() throws SAXException {

        try {

            Source source = new DOMSource(domDocument);

            File tmpFile = File.createTempFile(prefix, "tmp");

            Result resultat = new StreamResult(tmpFile);

            TransformerFactory fabrique = TransformerFactory.newInstance();
            Transformer transformer = fabrique.newTransformer();

            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            // transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");

            transformer.transform(source, resultat);

            /*
             * create BufferedReader
             */
            BufferedReader in = new BufferedReader(new FileReader(tmpFile));

            /*
             * create BufferedWriter
             */
            BufferedWriter out = new BufferedWriter(
                    new FileWriter(templateFile));

            out.write(buffer.toString());

            String line;
            while ((line = in.readLine()) != null) {
                out.write(line);
                out.write("\n");
            }

            out.close();
            in.close();

        } catch (TransformerConfigurationException e) {
            e.printStackTrace();
        } catch (TransformerException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
     *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String uri, String localName, String name,
            Attributes atts) throws SAXException {

        /*
         * register currentNode
         */
        if (currentNode == null) {
            currentNode = domRoot;
        } else {

            if (lastOp == START_ELEMENT) {
                currentNode = currentNode.getFirstChild();
            } else {
                currentNode = currentNode.getNextSibling();
            }

            if ((!(currentNode instanceof Element))) {
                do {
                    currentNode = currentNode.getNextSibling();
                } while (!(currentNode instanceof Element));
            }

            lastOp = START_ELEMENT;

            /**
             * Use case :
             *
             * <element att1=v1 att2=v2 />
             *
             * becomes
             *
             * #if( !$prefix-element-att1 ) #set($prefix-element-att1 = "v1")
             * #end" #if( !$prefix-element-att2 ) #set($prefix-element-att2 =
             * "v2") #end" <element att1=${prefix-element-att1}
             * att2=${prefix-element-att2} />
             *
             */

            for (int i = 0; i < atts.getLength(); i++) {

                String templateVar = format(name + "-" + atts.getLocalName(i));

                if (!templateVars.add(templateVar)) {
                    int idx = 1;
                    String tmp = templateVar;
                    do {
                        templateVar = tmp + "-" + idx;
                        idx++;
                    } while (!templateVars.add(templateVar));

                }

                buffer.append(getDefaultPattern(templateVar, atts.getValue(i)));
                buffer.append("\n");

                ((Element) currentNode).getAttributeNode(atts.getLocalName(i))
                        .setNodeValue("${" + templateVar + "}");

            }
        }

    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public void endElement(String uri, String localName, String name)
            throws SAXException {

        if (lastOp == END_ELEMENT) {
            currentNode = currentNode.getParentNode();
        }
        lastOp = END_ELEMENT;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length)
            throws SAXException {

        /**
         * This use case : <element> value </element>
         *
         * becomes
         *
         * #if( !$prefix-element ) #set($prefix-element = "value") #end"
         * <element> ${prefix-element} </element>
         */

        String value = new String(ch, start, length);

        if ((!value.contains(" ")) && (!value.contains("\n"))) {
            String templateVar = format(currentNode.getNodeName());
            templateVars.add(templateVar);

            buffer.append(getDefaultPattern(templateVar, value));
            buffer.append("\n");

            currentNode.setTextContent("${" + templateVar + "}");
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
     */
    public void setDocumentLocator(Locator locator) {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String,
     *      java.lang.String)
     */
    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
     */
    public void endPrefixMapping(String prefix) throws SAXException {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int)
     */
    public void ignorableWhitespace(char[] ch, int start, int length)
            throws SAXException {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String,
     *      java.lang.String)
     */
    public void processingInstruction(String target, String data)
            throws SAXException {
    }

    /*
     * (non-Javadoc)
     *
     * @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String)
     */
    public void skippedEntity(String name) throws SAXException {
    }

    // ------------------------------------------------------------------------
    // Main method for test
    // ------------------------------------------------------------------------

    public static void main(String[] args) {

        XMLTemplateGenerator xtg = new XMLTemplateGenerator("jonas_realm");

        File jgroupshaProps = new File(
                "/home/legrandj/Desktop/conf/jonas-realm.xml");

        try {

            xtg.generate(jgroupshaProps);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
